@@ -1,6 +1,7 @@
 # Link to mcdreforged.rtext
 # The advance text component class for Minecraft
-from mcdreforged.minecraft.rtext import RColor, RTextBase, RStyle, RText, RAction, RTextTranslation, RTextList
+from mcdreforged.minecraft.rtext.style import RColor, RStyle, RAction
+from mcdreforged.minecraft.rtext.text import RTextTranslation, RTextList, RTextBase, RText
 from mcdreforged.translation.translation_text import RTextMCDRTranslation
 
 __all__ = [
