@@ -10,7 +10,7 @@ from mcdreforged.command.builder.exception import LiteralNotMatch, UnknownComman
 	UnknownRootArgument, RequirementNotMet, IllegalNodeOperation, \
 	CommandError
 from mcdreforged.command.command_source import CommandSource
-from mcdreforged.utils import misc_util, tree_printer
+from mcdreforged.utils import misc_util, tree_printer, class_util
 from mcdreforged.utils.types import MessageText
 
 __SOURCE_CONTEXT_CALLBACK = Union[Callable[[], Any], Callable[[CommandSource], Any], Callable[[CommandSource, dict], Any]]
@@ -31,6 +31,11 @@ class _ErrorHandler(NamedTuple):
 	handled: bool
 
 
+class _Requirement(NamedTuple):
+	requirement: REQUIRES_CALLBACK
+	failure_message_getter: Optional[FAIL_MSG_CALLBACK]
+
+
 _ERROR_HANDLER_TYPE = Dict[Type[CommandError], _ErrorHandler]
 Self = TypeVar('Self', bound='AbstractNode')
 
@@ -47,8 +52,7 @@ class AbstractNode(ABC):
 		self._callback: Optional[RUNS_CALLBACK] = None
 		self._error_handlers: _ERROR_HANDLER_TYPE = {}
 		self._child_error_handlers: _ERROR_HANDLER_TYPE = {}
-		self._requirement: REQUIRES_CALLBACK = lambda source: True
-		self._requirement_failure_message_getter: Optional[FAIL_MSG_CALLBACK] = None
+		self._requirements: List[_Requirement] = []
 		self._redirect_node: Optional[AbstractNode] = None
 		self._suggestion_getter: SUGGESTS_CALLBACK = lambda: []
 
@@ -79,6 +83,7 @@ class AbstractNode(ABC):
 		"""
 		if self._redirect_node is not None:
 			raise IllegalNodeOperation('Redirected node is not allowed to add child nodes')
+		class_util.check_type(node, AbstractNode)
 		if isinstance(node, Literal):
 			for literal in node.literals:
 				self._children_literal[literal].append(node)
@@ -117,6 +122,7 @@ class AbstractNode(ABC):
 		:param func: A callable that accepts up to 2 arguments.
 			Argument list: :class:`~mcdreforged.command.command_source.CommandSource`, :class:`dict` (:class:`~mcdreforged.command.builder.common.CommandContext`)
 		"""
+		class_util.check_type(func, Callable)
 		self._callback = func
 		return self
 
@@ -131,6 +137,10 @@ class AbstractNode(ABC):
 		At this time if the *failure_message_getter* parameter is available, MCDR will invoke *failure_message_getter* to get the message string
 		of the ``RequirementNotMet`` exception, otherwise a default message will be used
 
+		.. versionadded:: v2.7.0
+			Multiple :meth:`requires` call results in an "and" combination of all given requirements,
+			i.e. the current command context is satisfied iif. all given requirements are satisfied
+
 		:param requirement: A callable that accepts up to 2 arguments and returns a bool.
 			Argument list: :class:`~mcdreforged.command.command_source.CommandSource`, :class:`dict` (:class:`~mcdreforged.command.builder.common.CommandContext`)
 		:param failure_message_getter: An optional callable that accepts up to 2 arguments and returns a str or a :class:`~mcdreforged.minecraft.rtext.text.RTextBase`.
@@ -138,12 +148,13 @@ class AbstractNode(ABC):
 
 		Example usages::
 
-			node.requires(lambda src: src.has_permission(3))  # Permission check
-			node.requires(lambda src, ctx: ctx['page_count'] <= get_max_page())  # Dynamic range check
-			node.requires(lambda src, ctx: is_legal(ctx['target']), lambda src, ctx: 'target {} is illegal'.format(ctx['target']))  # Customized failure message
+			node1.requires(lambda src: src.has_permission(3))  # Permission check
+			node2.requires(lambda src, ctx: ctx['page_count'] <= get_max_page())  # Dynamic range check
+			node3.requires(lambda src, ctx: is_legal(ctx['target']), lambda src, ctx: 'target {} is illegal'.format(ctx['target']))  # Customized failure message
 		"""
-		self._requirement = requirement
-		self._requirement_failure_message_getter = failure_message_getter
+		class_util.check_type(requirement, Callable)
+		class_util.check_type(failure_message_getter, [Callable, None])
+		self._requirements.append(_Requirement(requirement, failure_message_getter))
 		return self
 
 	def redirects(self: Self, redirect_node: 'AbstractNode') -> Self:
@@ -162,6 +173,7 @@ class AbstractNode(ABC):
 		"""
 		if self.has_children():
 			raise IllegalNodeOperation('Node with children nodes is not allowed to be redirected')
+		class_util.check_type(redirect_node, AbstractNode)
 		self._redirect_node = redirect_node
 		return self
 
@@ -185,6 +197,7 @@ class AbstractNode(ABC):
 		:param suggestion: A callable function which accepts up to 2 parameters and return an iterable of str indicating the current command suggestions.
 			Argument list: :class:`~mcdreforged.command.command_source.CommandSource`, :class:`dict` (:class:`~mcdreforged.command.builder.common.CommandContext`)
 		"""
+		class_util.check_type(suggestion, Callable)
 		self._suggestion_getter = suggestion
 		return self
 
@@ -201,6 +214,8 @@ class AbstractNode(ABC):
 		"""
 		if not issubclass(error_type, CommandError):
 			raise TypeError('error_type parameter should be a class inherited from CommandError, but class {} found'.format(error_type))
+		class_util.check_type(error_type, Type)
+		class_util.check_type(handler, Callable)
 		self._error_handlers[error_type] = _ErrorHandler(handler, handled)
 		return self
 
@@ -210,6 +225,8 @@ class AbstractNode(ABC):
 		"""
 		if not issubclass(error_type, CommandError):
 			raise TypeError('error_type parameter should be a class inherited from CommandError, but class {} found'.format(error_type))
+		class_util.check_type(error_type, Type)
+		class_util.check_type(handler, Callable)
 		self._child_error_handlers[error_type] = _ErrorHandler(handler, handled)
 		return self
 
@@ -267,8 +284,9 @@ class AbstractNode(ABC):
 		# make sure all passed CommandContext are copies
 		args = list(args)
 		for i in range(len(args)):
-			if isinstance(args[i], CommandContext):
-				args[i] = args[i].copy()
+			arg = args[i]
+			if isinstance(arg, CommandContext):
+				args[i] = arg.copy()
 
 		return callback(*args[:spec_args_len])
 
@@ -283,6 +301,16 @@ class AbstractNode(ABC):
 		self.__handle_error(error, context, self._error_handlers)
 		raise error
 
+	def __check_requirements(self, context: CommandContext) -> Optional[FAIL_MSG_CALLBACK]:
+		"""
+		:return: None: requirement check passed;
+		failure_message_getter: requirement check failed
+		"""
+		for req in self._requirements:
+			if not self.__smart_callback(req.requirement, context.source, context):
+				return req.failure_message_getter or (lambda: None)
+		return None
+
 	def _get_suggestions(self, context: CommandContext) -> Iterable[str]:
 		return self.__smart_callback(self._suggestion_getter, context.source, context)
 
@@ -299,11 +327,10 @@ class AbstractNode(ABC):
 			total_read = len(command) - len(next_remaining)  # type: int
 
 			with context.read_command(self, parse_result, total_read):
-				if self._requirement is not None:
-					if not self.__smart_callback(self._requirement, context.source, context):
-						getter = self._requirement_failure_message_getter or (lambda: None)
-						failure_message = self.__smart_callback(getter, context.source, context)
-						self.__raise_error(RequirementNotMet(context.command_read, context.command_read, failure_message), context)
+				getter = self.__check_requirements(context)
+				if getter is not None:  # requirement check failed
+					failure_message = self.__smart_callback(getter, context.source, context)
+					self.__raise_error(RequirementNotMet(context.command_read, context.command_read, failure_message), context)
 
 				# Parsing finished
 				if len(next_remaining) == 0:
@@ -378,12 +405,12 @@ class AbstractNode(ABC):
 			total_read = len(context.command) - len(next_remaining)  # type: int
 
 			with context.read_command(self, result, total_read):
-				if self._requirement is not None and not self.__smart_callback(self._requirement, context.source, context):
+				if self.__check_requirements(context) is not None:
 					return CommandSuggestions()
 
 				# Parsing finished
 				if len(next_remaining) == 0:
-					# total_read == success_read means DIVIDER does not exists at the end of the input string
+					# total_read == success_read means DIVIDER does not exist at the end of the input string
 					# in that case, ends at this current node
 					if success_read == total_read:
 						return self_suggestions()
@@ -471,7 +498,7 @@ class Literal(EntryNode):
 		return '|'.join(sorted(self.literals))
 
 	def suggests(self, suggestion: SUGGESTS_CALLBACK) -> 'AbstractNode':
-		raise IllegalNodeOperation('Literal node doe not support suggests')
+		raise IllegalNodeOperation('Literal node does not support suggests')
 
 	def parse(self, text):
 		arg = utils.get_element(text)
@@ -508,4 +535,3 @@ class ArgumentNode(AbstractNode, ABC):
 
 	def __repr__(self):
 		return '{}[name={}]'.format(self.__class__.__name__, self.__name)
-
