@@ -1,3 +1,4 @@
+from mcdreforged.handler.impl.abstract_minecraft_handler import AbstractMinecraftHandler
 from mcdreforged.handler.impl.basic_handler import BasicHandler
 from mcdreforged.handler.impl.beta18_handler import Beta18Handler
 from mcdreforged.handler.impl.bukkit14_handler import Bukkit14Handler
@@ -6,16 +7,17 @@ from mcdreforged.handler.impl.bungeecord_handler import BungeecordHandler
 from mcdreforged.handler.impl.cat_server_handler import CatServerHandler
 from mcdreforged.handler.impl.forge_handler import ForgeHandler
 from mcdreforged.handler.impl.vanilla_handler import VanillaHandler
+from mcdreforged.handler.impl.velocity_handler import VelocityHandler
 from mcdreforged.handler.impl.waterfall_handler import WaterfallHandler
 
 __all__ = [
 	'BasicHandler',
 
-	'VanillaHandler',
-	'BukkitHandler', 'Bukkit14Handler', 'CatServerHandler',
-	'ForgeHandler',
-	'Beta18Handler',
+	'AbstractMinecraftHandler',
 
-	'BungeecordHandler',
-	'WaterfallHandler'
+	'VanillaHandler', 'Beta18Handler', 'ForgeHandler',
+	'BukkitHandler', 'Bukkit14Handler', 'CatServerHandler',
+
+	'BungeecordHandler', 'WaterfallHandler',
+	'VelocityHandler'
 ]
