@@ -4,11 +4,12 @@ Including on_info and !!MCDR, !!help command
 """
 
 from mcdreforged.info_reactor.abstract_info_reactor import AbstractInfoReactor
+from mcdreforged.info_reactor.info import Info
 from mcdreforged.plugin.plugin_event import MCDRPluginEvents
 
 
 class GeneralReactor(AbstractInfoReactor):
-	def react(self, info):
+	def react(self, info: Info):
 		command_source = info.get_command_source()
 		if command_source is not None:
 			self.mcdr_server.command_manager.execute_command(info.content, command_source)
