@@ -12,7 +12,6 @@ import psutil
 from ruamel.yaml import YAMLError
 
 from mcdreforged.command.command_manager import CommandManager
-from mcdreforged.config import Config
 from mcdreforged.constants import core_constant
 from mcdreforged.executor.console_handler import ConsoleHandler
 from mcdreforged.executor.task_executor import TaskExecutor
@@ -22,6 +21,7 @@ from mcdreforged.handler.server_handler_manager import ServerHandlerManager
 from mcdreforged.info_reactor.info import Info
 from mcdreforged.info_reactor.info_reactor_manager import InfoReactorManager
 from mcdreforged.info_reactor.server_information import ServerInformation
+from mcdreforged.mcdr_config import MCDReforgedConfig
 from mcdreforged.mcdr_state import ServerState, MCDReforgedState, MCDReforgedFlag
 from mcdreforged.minecraft.rcon.rcon_manager import RconManager
 from mcdreforged.permission.permission_manager import PermissionManager
@@ -50,6 +50,7 @@ class MCDReforgedServer:
 		self.flags = MCDReforgedFlag.NONE
 		self.starting_server_lock = Lock()  # to prevent multiple start_server() call
 		self.stop_lock = Lock()  # to prevent multiple stop() call
+		self.config_change_lock = Lock()
 
 		# will be assigned in on_config_changed()
 		self.encoding_method = None  # type: Optional[str]
@@ -57,7 +58,7 @@ class MCDReforgedServer:
 
 		# --- Constructing fields --- #
 		self.logger = MCDReforgedLogger()
-		self.config = Config(self.logger)
+		self.config = MCDReforgedConfig(self.logger)
 		self.permission_manager = PermissionManager(self)
 		self.basic_server_interface = ServerInterface(self)
 		self.task_executor = TaskExecutor(self)
@@ -114,7 +115,7 @@ class MCDReforgedServer:
 		# load_permission_file: permission
 		# config change will lead to creating plugin folders
 		loading_success = \
-			load('configuration', lambda: self.load_config(allowed_missing_file=False, echo=not initialize_environment)) and \
+			load('configuration', lambda: self.load_config(allowed_missing_file=False, log=not initialize_environment)) and \
 			load('permission', lambda: self.permission_manager.load_permission_file(allowed_missing_file=False))
 		if file_missing:
 			self.__on_file_missing()
@@ -122,7 +123,7 @@ class MCDReforgedServer:
 		if not loading_success:
 			return
 
-		# MCDR environment has been setup, so continue creating default folders and loading stuffs
+		# MCDR environment has been set up, so continue creating default folders and loading stuffs
 		self.logger.set_file(core_constant.LOGGING_FILE)  # will create logs/ folder
 		self.plugin_manager.touch_directory()  # will create config/ folder
 
@@ -185,47 +186,48 @@ class MCDReforgedServer:
 	#          Loaders
 	# --------------------------
 
-	def load_config(self, *, allowed_missing_file: bool = True, echo: bool = True):
+	def load_config(self, *, allowed_missing_file: bool = True, log: bool = True):
 		has_missing = self.config.read_config(allowed_missing_file)
-		# load the language first to make sure tr() is available
-		self.on_config_changed(echo)
-		if echo and has_missing:
+		# load the language before warning missing config to make sure translation is available
+		self.on_config_changed(log=log)
+		if log and has_missing:
 			for line in self.tr('config.missing_config').splitlines():
 				self.logger.warning(line)
 
-	def on_config_changed(self, echo: bool):
-		MCColoredFormatter.console_color_disabled = self.config['disable_console_color']
-		self.logger.set_debug_options(self.config['debug'])
-		if echo and self.config.is_debug_on():
-			self.logger.info(self.tr('mcdr_server.on_config_changed.debug_mode_on'))
+	def on_config_changed(self, *, log: bool):
+		with self.config_change_lock:
+			MCColoredFormatter.console_color_disabled = self.config['disable_console_color']
+			self.logger.set_debug_options(self.config['debug'])
+			if log and self.config.is_debug_on():
+				self.logger.info(self.tr('mcdr_server.on_config_changed.debug_mode_on'))
 
-		self.translation_manager.set_language(self.config['language'])
-		if echo:
-			self.logger.info(self.tr('mcdr_server.on_config_changed.language_set', self.config['language']))
+			self.translation_manager.set_language(self.config['language'])
+			if log:
+				self.logger.info(self.tr('mcdr_server.on_config_changed.language_set', self.config['language']))
 
-		self.encoding_method = self.config['encoding'] if self.config['encoding'] is not None else sys.getdefaultencoding()
-		self.decoding_method = self.config['decoding'] if self.config['decoding'] is not None else locale.getpreferredencoding()
-		if echo:
-			self.logger.info(self.tr('mcdr_server.on_config_changed.encoding_decoding_set', self.encoding_method, self.decoding_method))
+			self.encoding_method = self.config['encoding'] if self.config['encoding'] is not None else sys.getdefaultencoding()
+			self.decoding_method = self.config['decoding'] if self.config['decoding'] is not None else locale.getpreferredencoding()
+			if log:
+				self.logger.info(self.tr('mcdr_server.on_config_changed.encoding_decoding_set', self.encoding_method, self.decoding_method))
 
-		self.plugin_manager.set_plugin_directories(self.config['plugin_directories'])
-		if echo:
-			self.logger.info(self.tr('mcdr_server.on_config_changed.plugin_directories_set', self.encoding_method, self.decoding_method))
-			for directory in self.plugin_manager.plugin_directories:
-				self.logger.info('- {}'.format(directory))
+			self.plugin_manager.set_plugin_directories(self.config['plugin_directories'])
+			if log:
+				self.logger.info(self.tr('mcdr_server.on_config_changed.plugin_directories_set', self.encoding_method, self.decoding_method))
+				for directory in self.plugin_manager.plugin_directories:
+					self.logger.info('- {}'.format(directory))
 
-		self.reactor_manager.register_reactors(self.config['custom_info_reactors'])
+			self.reactor_manager.register_reactors(self.config['custom_info_reactors'])
 
-		self.server_handler_manager.register_handlers(self.config['custom_handlers'])
-		self.server_handler_manager.set_handler(self.config['handler'])
-		if echo:
-			self.logger.info(self.tr('mcdr_server.on_config_changed.handler_set', self.config['handler']))
+			self.server_handler_manager.register_handlers(self.config['custom_handlers'])
+			self.server_handler_manager.set_handler(self.config['handler'])
+			if log:
+				self.logger.info(self.tr('mcdr_server.on_config_changed.handler_set', self.config['handler']))
 
-		self.connect_rcon()
+			self.connect_rcon()
 
 	def load_plugins(self):
-		self.plugin_manager.refresh_all_plugins()
-		self.logger.info(self.plugin_manager.last_operation_result.to_rtext(show_path=True))
+		future = self.plugin_manager.refresh_all_plugins()
+		self.logger.info(future.get().to_rtext(self, show_path=True))
 
 	def on_plugin_registry_changed(self):
 		self.command_manager.clear_command()
@@ -413,7 +415,7 @@ class MCDReforgedServer:
 
 	def __on_server_stop(self):
 		return_code = self.process.poll()
-		self.logger.info(self.tr('mcdr_server.on_server_stop.show_stopcode', return_code))
+		self.logger.info(self.tr('mcdr_server.on_server_stop.show_return_code', return_code))
 		try:
 			self.process.stdin.close()
 		except Exception as e:
