@@ -7,7 +7,7 @@ from typing import List, Dict, TYPE_CHECKING, Optional, Union
 from mcdreforged.minecraft.rtext.text import RTextBase, RText
 from mcdreforged.plugin.meta.version import Version, VersionParsingError, VersionRequirement
 from mcdreforged.translation.translation_text import RTextMCDRTranslation
-from mcdreforged.utils import translation_util, misc_util, class_util
+from mcdreforged.utils import translation_util, class_util
 from mcdreforged.utils.types import TranslationLanguageDict
 
 if TYPE_CHECKING:
@@ -87,18 +87,18 @@ class Metadata:
 				warn('{}, use fallback id {} instead'.format(use_fallback_id_reason, self.id))
 			else:
 				raise ValueError('Plugin id not found in metadata')
-		misc_util.check_type(self.id, str)
+		class_util.check_type(self.id, str)
 
 		self.name = data.get('name', self.id)
 		if isinstance(self.name, RTextBase):
 			self.name = self.name.to_plain_text()
-		misc_util.check_type(self.name, str)
+		class_util.check_type(self.name, str)
 
 		description = data.get('description')
 		if isinstance(description, RTextBase):
 			description = description.to_plain_text()
 		self.description = description
-		misc_util.check_type(self.description, [type(None), str, dict])
+		class_util.check_type(self.description, [None, str, dict])
 
 		self.author = data.get('author')
 		if isinstance(self.author, str):
@@ -108,10 +108,10 @@ class Metadata:
 				self.author[i] = str(self.author[i])
 			if len(self.author) == 0:
 				self.author = None
-		misc_util.check_type(self.author, [type(None), list])
+		class_util.check_type(self.author, [None, list])
 
 		self.link = data.get('link')
-		misc_util.check_type(self.link, [type(None), str])
+		class_util.check_type(self.link, [None, str])
 
 		version_str = data.get('version')
 		if version_str:
@@ -121,7 +121,7 @@ class Metadata:
 				warn('Version "{}" of {} is invalid ({}), ignore and use fallback version instead {}'.format(version_str, plugin_name_text, e, self.FALLBACK_VERSION))
 				version_str = None
 		else:
-			warn('{} doesn\'t specific a version, use fallback version {}'.format(plugin_name_text, self.FALLBACK_VERSION))
+			warn("{} doesn't specific a version, use fallback version {}".format(plugin_name_text, self.FALLBACK_VERSION))
 		if version_str is None:
 			self.version = Version(self.FALLBACK_VERSION)
 
@@ -135,15 +135,15 @@ class Metadata:
 				))
 
 		self.entrypoint = data.get('entrypoint', self.id)
-		misc_util.check_type(self.entrypoint, str)
+		class_util.check_type(self.entrypoint, str)
 		# entrypoint module should be inside the plugin module
 		if self.entrypoint != self.id and not self.entrypoint.startswith(self.id + '.'):
 			raise ValueError('Invalid entry point "{}" for plugin id "{}"'.format(self.entrypoint, self.id))
 
 		self.archive_name = data.get('archive_name')
 		self.resources = data.get('resources', [])
-		misc_util.check_type(self.archive_name, [type(None), str])
-		misc_util.check_type(self.resources, list)
+		class_util.check_type(self.archive_name, [None, str])
+		class_util.check_type(self.resources, list)
 
 	def __repr__(self):
 		return class_util.represent(self)
@@ -164,6 +164,8 @@ class Metadata:
 	def get_description_rtext(self) -> RTextBase:
 		"""
 		Return a translated plugin description in :class:`RText <mcdreforged.minecraft.rtext.text.RTextBase>`
+
+		.. versionadded:: v2.1.2
 		"""
 		if isinstance(self.description, str):
 			return RText(self.description)
