@@ -3,6 +3,7 @@ Plugin management
 """
 import collections
 import os
+import queue
 import threading
 from contextlib import contextmanager
 from typing import Callable, Dict, Optional, Any, Tuple, List, TYPE_CHECKING, Deque
@@ -19,7 +20,8 @@ from mcdreforged.plugin.plugin_thread import PluginThreadPool
 from mcdreforged.plugin.type.permanent_plugin import PermanentPlugin
 from mcdreforged.plugin.type.plugin import AbstractPlugin, PluginState
 from mcdreforged.plugin.type.regular_plugin import RegularPlugin
-from mcdreforged.utils import file_util, string_util, misc_util
+from mcdreforged.utils import file_util, string_util, misc_util, class_util
+from mcdreforged.utils.future import Future
 from mcdreforged.utils.logger import DebugOption
 from mcdreforged.utils.thread_local_storage import ThreadLocalStorage
 
@@ -35,23 +37,23 @@ class PluginManager:
 		self.mcdr_server = mcdr_server
 		self.logger = mcdr_server.logger
 
-		# id -> Plugin plugin storage
-		self.plugins = {}   # type: Dict[str, AbstractPlugin]
+		# plugin storage, id -> Plugin
+		self.plugins: Dict[str, AbstractPlugin] = {}
 		# file_path -> id mapping
-		self.plugin_file_path = {}  # type: Dict[str, str]
+		self.plugin_file_path: Dict[str, str] = {}
 		# storage for event listeners, help messages and commands
 		self.registry_storage = PluginRegistryStorage(self)
 
-		self.last_operation_result = PluginOperationResult(self)
-
 		# not used currently
 		self.thread_pool = PluginThreadPool(self.mcdr_server, max_thread=core_constant.PLUGIN_THREAD_POOL_SIZE)
 
 		# thread local storage, to store current plugin
 		self.__tls = ThreadLocalStorage()
 
-		# plugin manipulation lock
+		# plugin manipulation logics
 		self.__mani_lock = threading.RLock()
+		self.__mani_thread: Optional[threading.Thread] = None
+		self.__mani_queue = queue.Queue()
 
 	@classmethod
 	def touch_directory(cls):
@@ -158,14 +160,13 @@ class PluginManager:
 	#   Single Plugin Operations
 	# ----------------------------
 
-	def __load_plugin(self, file_path):
+	def __load_plugin(self, file_path: str) -> Optional[RegularPlugin]:
 		"""
 		Try to load a plugin from the given file
 		If succeeds, add the plugin to the plugin list, the plugin state will be set to LOADED
 		If fails, nothing will happen
-		:param str file_path: The path to the plugin file, a *.py
+		:param file_path: The path to the plugin file
 		:return: the new plugin instance if succeeds, otherwise None
-		:rtype: RegularPlugin or None
 		"""
 		plugin = plugin_factory.create_regular_plugin(self, file_path)
 		try:
@@ -189,14 +190,12 @@ class PluginManager:
 				plugin.remove()  # quickly remove this plugin
 				return None
 
-	def __unload_plugin(self, plugin):
+	def __unload_plugin(self, plugin: AbstractPlugin) -> bool:
 		"""
 		Try to load a plugin from the given file
 		Whether it succeeds or not, the plugin instance will be removed from the plugin list
 		The plugin state will be set to UNLOADING
-		:param AbstractPlugin plugin: The plugin instance to be unloaded
 		:return: If there's an exception during plugin unloading
-		:rtype: bool
 		"""
 		try:
 			plugin.unload()
@@ -207,21 +206,18 @@ class PluginManager:
 			ret = False
 		else:
 			self.logger.info(self.mcdr_server.tr('plugin_manager.unload_plugin.success', plugin.get_name()))
+			plugin.receive_event(MCDRPluginEvents.PLUGIN_UNLOADED, ())
 			ret = True
 		finally:
 			self.__remove_plugin(plugin)
 		return ret
 
-	def __reload_plugin(self, plugin):
+	def __reload_plugin(self, plugin: AbstractPlugin) -> bool:
 		"""
-		Try to reload an existed plugin
+		Try to reload an existed and unloaded plugin
 		If fails, unload the plugin and then the plugin state will be set to UNLOADED
-		:param AbstractPlugin plugin: The plugin instance to be reloaded
 		:return: If the plugin reloads successfully without error
-		:rtype: bool
 		"""
-		plugin.receive_event(MCDRPluginEvents.PLUGIN_UNLOADED, ())
-		self.__remove_plugin(plugin)
 		try:
 			plugin.reload()
 		except:
@@ -260,9 +256,9 @@ class PluginManager:
 				self.logger.warning('Plugin directory "{}" not found'.format(plugin_directory))
 		return paths
 
-	def __collect_and_process_new_plugins(self, filter: Callable[[str], bool], *, possible_paths: Optional[List[str]] = None) -> SingleOperationResult:
+	def __collect_and_process_new_plugins(self, filter_: Callable[[str], bool], *, possible_paths: Optional[List[str]] = None) -> SingleOperationResult:
 		"""
-		:param filter: A str predicate function for testing if the plugin file path is acceptable
+		:param filter_: A str predicate function for testing if the plugin file path is acceptable
 		:param possible_paths: Optional. If you have already done self.__collect_possible_plugin_file_paths() before,
 		you can pass the previous result as the argument to reuse that, so less time cost
 		"""
@@ -271,7 +267,7 @@ class PluginManager:
 
 		result = SingleOperationResult()
 		for file_path in possible_paths:
-			if not self.contains_plugin_file(file_path) and filter(file_path):
+			if not self.contains_plugin_file(file_path) and filter_(file_path):
 				plugin = self.__load_plugin(file_path)
 				if plugin is None:
 					result.fail(file_path)
@@ -279,65 +275,105 @@ class PluginManager:
 					result.succeed(plugin)
 		return result
 
-	def __collect_and_remove_plugins(self, filter: Callable[[RegularPlugin], bool], specific: Optional[RegularPlugin] = None) -> SingleOperationResult:
-		result = SingleOperationResult()
+	def __collect_regular_plugins(
+			self, select_filter: Callable[[RegularPlugin], bool], specific: Optional[RegularPlugin],
+			operation_name: str = 'operate', collect_filter: Callable[[RegularPlugin], bool] = lambda p: True
+	) -> List[RegularPlugin]:
+		"""
+		collected = selected + affected
+		:return: A list of plugin in topo order
+		"""
 		plugin_list = self.get_regular_plugins() if specific is None else [specific]
-		for plugin in plugin_list:
-			if filter(plugin):
-				result.record(plugin, self.__unload_plugin(plugin))
-		return result
+		selected_plugins = list(filter(select_filter, plugin_list))
+		selected_plugins_set = set(selected_plugins)
 
-	def __reload_ready_plugins(self, filter: Callable[[RegularPlugin], bool], specific: Optional[RegularPlugin] = None) -> SingleOperationResult:
-		result = SingleOperationResult()
-		plugin_list = self.get_regular_plugins() if specific is None else [specific]
-		for plugin in plugin_list:
-			if plugin.in_states({PluginState.READY}) and filter(plugin):
-				result.record(plugin, self.__reload_plugin(plugin))
-		return result
-
-	def __check_plugin_dependencies(self) -> SingleOperationResult:
-		result = SingleOperationResult()
 		walker = DependencyWalker(self)
-		walk_result = walker.walk()
-		for item in walk_result:
-			plugin = self.plugins.get(item.plugin_id)  # should be not None
-			result.record(plugin, item.success)
-			if not item.success:
-				self.logger.error(self.mcdr_server.tr('plugin_manager.check_plugin_dependencies.item_failed', plugin, item.reason))
-				self.__unload_plugin(plugin)
-		self.logger.debug(self.mcdr_server.tr('plugin_manager.check_plugin_dependencies.topo_order'), option=DebugOption.PLUGIN)
-		for plugin in result.success_list:
-			self.logger.debug('- {}'.format(plugin), option=DebugOption.PLUGIN)
-		# the success list order matches the dependency topo order
+		walker.walk()
+		affected_plugin_ids: List[str] = []
+
+		for plugin in selected_plugins:
+			affected_plugin_ids.extend(walker.get_children(plugin.get_id()))
+
+		collected_plugins: List[RegularPlugin] = []  # plugins, in topo order
+		for plugin_id in sorted(misc_util.unique_list(affected_plugin_ids), key=walker.get_topo_order):
+			plugin = self.get_regular_plugin_from_id(plugin_id)
+			assert plugin is not None
+			if collect_filter(plugin):
+				collected_plugins.append(plugin)
+
+		affected_plugins = list(filter(lambda plg: plg not in selected_plugins_set, collected_plugins))
+		if self.logger.should_log_debug(DebugOption.PLUGIN):
+			self.logger.debug('Collected {}x plugin'.format(len(collected_plugins)) + ':' if len(collected_plugins) > 0 else '', no_check=True)
+			if len(collected_plugins) > 0:
+				self.logger.debug('- selected {}x: {}'.format(len(selected_plugins), ', '.join(map(str, selected_plugins))), no_check=True)
+				self.logger.debug('- affected {}x: {}'.format(len(affected_plugins), ', '.join(map(str, affected_plugins))), no_check=True)
+		if len(affected_plugins) > 0:
+			self.logger.info('Collected {}x extra plugins for {} due to dependency associations: {}'.format(len(affected_plugins), operation_name, ', '.join(map(str, affected_plugins))))
+
+		return collected_plugins
+
+	def __unload_given_plugins(self, filter_: Callable[[RegularPlugin], bool], specific: Optional[RegularPlugin] = None) -> SingleOperationResult:
+		affected_plugins = self.__collect_regular_plugins(filter_, specific, 'unload')
+		result = SingleOperationResult()
+		for plugin in affected_plugins:
+			result.record(plugin, self.__unload_plugin(plugin))
+		return result
+
+	def __reload_ready_plugins(self, filter_: Callable[[RegularPlugin], bool], specific: Optional[RegularPlugin] = None) -> SingleOperationResult:
+		def state_check(plg: RegularPlugin):
+			return plg.in_states({PluginState.READY})
+
+		affected_plugins = self.__collect_regular_plugins(lambda plg: state_check(plg) and filter_(plg), specific, 'reload', state_check)
+		result = SingleOperationResult()
+
+		# child first, parent last
+		for plugin in reversed(affected_plugins):
+			self.__unload_plugin(plugin)
+
+		# parent first, child last
+		# actually the order is not sensitive here, but why not
+		for plugin in affected_plugins:
+			result.record(plugin, self.__reload_plugin(plugin))
 		return result
 
 	# ---------------------------
 	#   Multi-Plugin Operations
 	# ---------------------------
 
-	# MCDR 0.x behavior for reload
-	# 1. reload existed (and changed) plugins. if reload fail unload it
-	# 2. load new plugins, and ignore those plugins which just got unloaded because reloading failed
-	# 3. unload removed plugins
-	# 4. call on_load for new loaded plugins
-
 	# Current behavior for plugin operations
-	# 1. Actual plugin operations
-	#   For plugin refresh
-	#      1. Unload plugins whose file is removed
-	#      2. Load new plugins
-	#      3. Reload existed (and matches filter) plugins whose state is ready. if reload fail unload it
 	#
-	#   For single plugin operation
-	#      1. Enable / disable plugin
+	# Steps to take
+	#   load: load, check dept, finalization
+	#   unload: unload, finalization
+	#   reload: unload, load, check dept, finalization
 	#
-	# 2. Plugin Processing  (method __post_plugin_process)
-	#      1. Check dependencies, unload plugins that has dependencies not satisfied
-	#      2. Call on_unload for unloaded plugins
-	#      3. Call on_load for new / reloaded plugins by topo order
-	#    Dispatch PLUGIN_UNLOADED before PLUGIN_LOADED, so file renamed plugin keeps the same event order (unload -> load) with reloaded plugin
+	# Common path: unload, load, check dept, finalization
+	#
+	# In detailed:
+	# 1. Do unload in reversed topo order:
+	#   a. dispatch plugin unload event
+	#   b. unload plugin object
+	#
+	# 2. Load plugins
+	# 3. Check dependencies
+	#
+	# 4. Finalization (:meth:`__finalization_plugin_manipulation`)
+	#   a. Remove / Get-ready plugin objects
+	#   b. Do load in topo order: dispatch on_load event
+	#   c. Update registry
+	#   d. Sort plugins
+	#
+	# When to dispatch load/unload event:
+	# - load: after dept check
+	# - unload: immediately
+	#
+	# New sync operations will be queued and delayed
+	#   see :meth:`__run_manipulation`
 
-	def __post_plugin_process(self, load_result=None, unload_result=None, reload_result=None):
+	def __finalization_plugin_manipulation(
+			self, load_result: Optional[SingleOperationResult] = None, unload_result: Optional[SingleOperationResult] = None,
+			reload_result: Optional[SingleOperationResult] = None
+	) -> PluginOperationResult:
 		"""
 		When returns, all events / operations are finished processing
 		"""
@@ -348,40 +384,48 @@ class PluginManager:
 		if reload_result is None:
 			reload_result = SingleOperationResult()
 
-		dependency_check_result = self.__check_plugin_dependencies()
-		self.last_operation_result.record(load_result, unload_result, reload_result, dependency_check_result)
+		walker = DependencyWalker(self)
+		walk_result = walker.walk()
+		dependency_check_result = SingleOperationResult()  # topo order in success_list
+		for item in walk_result:
+			plugin = self.plugins[item.plugin_id]
+			dependency_check_result.record(plugin, item.success)
+			if not item.success:
+				self.logger.error(self.mcdr_server.tr('plugin_manager.check_plugin_dependencies.item_failed', plugin, item.reason))
+				self.__unload_plugin(plugin)
+
+		self.logger.debug(self.mcdr_server.tr('plugin_manager.check_plugin_dependencies.topo_order'), option=DebugOption.PLUGIN)
+		for plugin in dependency_check_result.success_list:
+			self.logger.debug('- {}'.format(plugin), option=DebugOption.PLUGIN)
 
 		# Expected plugin states:
-		# 					success_list		fail_list
-		# load_result		LOADED				N/A
-		# unload_result		UNLOADING			UNLOADING
-		# reload_result		READY				UNLOADING
-		# dep_chk_result	LOADED / READY		UNLOADING
+		#                   success_list        fail_list
+		# load_result       LOADED              N/A
+		# unload_result     UNLOADING           UNLOADING
+		# reload_result     LOADED              UNLOADING
+		# dep_chk_result    LOADED              UNLOADING
 
-		self.registry_storage.clear()  # in case plugin invokes dispatch_event during on_load. dont let them trigger listeners
+		self.registry_storage.clear()  # in case plugin invokes dispatch_event during on_load. don't let them trigger listeners
 
 		newly_loaded_plugins = {*load_result.success_list, *reload_result.success_list}
 		to_be_removed_plugins = unload_result.success_list + unload_result.failed_list + reload_result.failed_list + dependency_check_result.failed_list
 
-		# collect removed plugin module instance
+		# collect removed plugin module instance in case e.g. reloading when a plugin file was renamed
+		# so its on_load event can still have the old entry module
 		removed_plugins_module_instance = {}
 		for plugin in to_be_removed_plugins:
 			if isinstance(plugin, RegularPlugin):
 				removed_plugins_module_instance[plugin.get_id()] = plugin.entry_module_instance
 
+		# do remove
+		for plugin in to_be_removed_plugins:
+			plugin.remove()
+
 		# get ready
 		for plugin in dependency_check_result.success_list:
 			if plugin in newly_loaded_plugins:
 				plugin.ready()
 
-		# PLUGIN_UNLOADED event
-		for plugin in to_be_removed_plugins:
-			plugin.assert_state({PluginState.UNLOADING})
-			# plugins might just be newly loaded but failed on dependency check, dont dispatch event to them
-			if plugin not in newly_loaded_plugins:
-				plugin.receive_event(MCDRPluginEvents.PLUGIN_UNLOADED, ())
-			plugin.remove()
-
 		# PLUGIN_LOADED event
 		for plugin in dependency_check_result.success_list:
 			if plugin in newly_loaded_plugins:
@@ -398,6 +442,8 @@ class PluginManager:
 		self.__update_registry()
 		self.__sort_plugins_by_id()
 
+		return PluginOperationResult(load_result, unload_result, reload_result, dependency_check_result)
+
 	def __sort_plugins_by_id(self):
 		self.plugins = dict(sorted(map(tuple, self.plugins.items()), key=lambda item: item[0]))
 
@@ -408,61 +454,110 @@ class PluginManager:
 		self.registry_storage.arrange()
 		self.mcdr_server.on_plugin_registry_changed()
 
-	def __refresh_plugins(self, reload_filter: Callable[[RegularPlugin], bool]):
+	def __refresh_plugins(self, reload_filter: Callable[[RegularPlugin], bool]) -> PluginOperationResult:
 		possible_paths = self.__collect_possible_plugin_file_paths()
 		possible_paths_set = set(possible_paths)
-		unload_result = self.__collect_and_remove_plugins(lambda plugin: not plugin.plugin_exists() or plugin.plugin_path not in possible_paths_set)
+		unload_result = self.__unload_given_plugins(lambda plugin: not plugin.plugin_exists() or plugin.plugin_path not in possible_paths_set)
 		load_result = self.__collect_and_process_new_plugins(lambda fp: True, possible_paths=possible_paths)
 		reload_result = self.__reload_ready_plugins(reload_filter)
-		self.__post_plugin_process(load_result, unload_result, reload_result)
+		return self.__finalization_plugin_manipulation(load_result, unload_result, reload_result)
 
 	# --------------
 	#   Interfaces
 	# --------------
 
-	def load_plugin(self, file_path: str):
+	def __run_manipulation(self, action: Callable[[], PluginOperationResult]) -> Future[PluginOperationResult]:
+		"""
+		Async manipulations: Blocked and wait via self.__mani_lock
+
+		Sync manipulations: Run directly for the 1st action, store in queue for other actions.
+		The delayed actions in queue will be executed after the 1st action is done
+
+		:return: A future to the plugin operation result. The future is finished
+		iif. it's the 1st manipulation call in the current thread's call chain
+		"""
 		with self.__mani_lock:
+			future: Future[PluginOperationResult] = Future()
+			self.__mani_queue.put((action, future))
+			if self.__mani_thread != threading.current_thread():
+				self.__mani_thread = threading.current_thread()
+				try:
+					while True:
+						try:
+							queued_action, queued_future = self.__mani_queue.get_nowait()
+						except queue.Empty:
+							break
+						else:
+							result = queued_action()
+							queued_future.set_result(result)
+				finally:
+					self.__mani_thread = None
+			else:
+				self.logger.debug('Detected chained sync plugin manipulation, queueing', option=DebugOption.PLUGIN)
+
+			return future
+
+	def load_plugin(self, file_path: str) -> Future[PluginOperationResult]:
+		def load_plugin_action() -> PluginOperationResult:
 			self.logger.info(self.mcdr_server.tr('plugin_manager.load_plugin.entered', file_path))
 			load_result = self.__collect_and_process_new_plugins(lambda fp: fp == file_path)
-			self.__post_plugin_process(load_result=load_result)
+			return self.__finalization_plugin_manipulation(load_result=load_result)
 
-	def unload_plugin(self, plugin: RegularPlugin):
-		with self.__mani_lock:
+		class_util.check_type(file_path, str)
+		return self.__run_manipulation(load_plugin_action)
+
+	def unload_plugin(self, plugin: RegularPlugin) -> Future[PluginOperationResult]:
+		def unload_plugin_action() -> PluginOperationResult:
 			self.logger.info(self.mcdr_server.tr('plugin_manager.unload_plugin.entered', plugin))
-			unload_result = self.__collect_and_remove_plugins(lambda plg: True, specific=plugin)
-			self.__post_plugin_process(unload_result=unload_result)
+			unload_result = self.__unload_given_plugins(lambda plg: True, specific=plugin)
+			return self.__finalization_plugin_manipulation(unload_result=unload_result)
 
-	def reload_plugin(self, plugin: RegularPlugin):
-		with self.__mani_lock:
+		class_util.check_type(plugin, RegularPlugin)
+		return self.__run_manipulation(unload_plugin_action)
+
+	def reload_plugin(self, plugin: RegularPlugin) -> Future[PluginOperationResult]:
+		def reload_plugin_action() -> PluginOperationResult:
 			self.logger.info(self.mcdr_server.tr('plugin_manager.reload_plugin.entered', plugin))
 			reload_result = self.__reload_ready_plugins(lambda plg: True, specific=plugin)
-			self.__post_plugin_process(reload_result=reload_result)
+			return self.__finalization_plugin_manipulation(reload_result=reload_result)
 
-	def enable_plugin(self, file_path: str):
-		with self.__mani_lock:
-			self.logger.info(self.mcdr_server.tr('plugin_manager.enable_plugin.entered', file_path))
-			new_file_path = string_util.remove_suffix(file_path, plugin_constant.DISABLED_PLUGIN_FILE_SUFFIX)
-			if plugin_factory.is_disabled_plugin(file_path):
-				os.rename(file_path, new_file_path)
-				self.load_plugin(new_file_path)
+		class_util.check_type(plugin, RegularPlugin)
+		return self.__run_manipulation(reload_plugin_action)
 
-	def disable_plugin(self, plugin: RegularPlugin):
-		with self.__mani_lock:
-			self.logger.info(self.mcdr_server.tr('plugin_manager.disable_plugin.entered', plugin))
-			self.unload_plugin(plugin)
+	def enable_plugin(self, file_path: str) -> Future[PluginOperationResult]:
+		class_util.check_type(file_path, str)
+		self.logger.info(self.mcdr_server.tr('plugin_manager.enable_plugin.entered', file_path))
+		new_file_path = string_util.remove_suffix(file_path, plugin_constant.DISABLED_PLUGIN_FILE_SUFFIX)
+		if plugin_factory.is_disabled_plugin(file_path):
+			os.rename(file_path, new_file_path)
+			return self.load_plugin(new_file_path)
+		else:
+			return Future.completed(PluginOperationResult.of_empty())
+
+	def disable_plugin(self, plugin: RegularPlugin) -> Future[PluginOperationResult]:
+		def done_callback(_: PluginOperationResult):
 			if plugin.plugin_exists():
 				os.rename(plugin.plugin_path, plugin.plugin_path + plugin_constant.DISABLED_PLUGIN_FILE_SUFFIX)
 
-	def refresh_all_plugins(self):
-		with self.__mani_lock:
+		self.logger.info(self.mcdr_server.tr('plugin_manager.disable_plugin.entered', plugin))
+		future = self.unload_plugin(plugin)
+		future.add_done_callback(done_callback)
+		return future
+
+	def refresh_all_plugins(self) -> Future[PluginOperationResult]:
+		def refresh_all_plugins_action() -> PluginOperationResult:
 			self.logger.info(self.mcdr_server.tr('plugin_manager.refresh_all_plugins.entered'))
 			return self.__refresh_plugins(lambda plg: True)
 
-	def refresh_changed_plugins(self):
-		with self.__mani_lock:
+		return self.__run_manipulation(refresh_all_plugins_action)
+
+	def refresh_changed_plugins(self) -> Future[PluginOperationResult]:
+		def refresh_changed_plugins_action() -> PluginOperationResult:
 			self.logger.info(self.mcdr_server.tr('plugin_manager.refresh_changed_plugins.entered'))
 			return self.__refresh_plugins(lambda plg: plg.file_changed())
 
+		return self.__run_manipulation(refresh_changed_plugins_action)
+
 	# ----------------
 	#   Plugin Event
 	# ----------------
