@@ -15,15 +15,16 @@ from mcdreforged.mcdr_state import MCDReforgedFlag
 from mcdreforged.permission.permission_level import PermissionLevel, PermissionParam
 from mcdreforged.plugin import plugin_factory
 from mcdreforged.plugin.meta.metadata import Metadata
-from mcdreforged.plugin.operation_result import SingleOperationResult, PluginOperationResult
+from mcdreforged.plugin.operation_result import PluginOperationResult, PluginResultType
 from mcdreforged.plugin.plugin_event import EventListener, LiteralEvent, PluginEvent, MCDRPluginEvents
 from mcdreforged.plugin.plugin_registry import DEFAULT_LISTENER_PRIORITY, HelpMessage
 from mcdreforged.plugin.type.multi_file_plugin import MultiFilePlugin
 from mcdreforged.plugin.type.plugin import AbstractPlugin
 from mcdreforged.preference.preference_manager import PreferenceItem
 from mcdreforged.translation.translation_text import RTextMCDRTranslation
-from mcdreforged.utils import misc_util, file_util
+from mcdreforged.utils import misc_util, file_util, class_util
 from mcdreforged.utils.exception import IllegalCallError
+from mcdreforged.utils.future import Future
 from mcdreforged.utils.logger import MCDReforgedLogger, DebugOption
 from mcdreforged.utils.serializer import Serializable
 from mcdreforged.utils.types import MessageText, TranslationKeyDictRich, TranslationKeyDictNested
@@ -34,7 +35,7 @@ if TYPE_CHECKING:
 	from mcdreforged.plugin.plugin_manager import PluginManager
 	from mcdreforged.plugin.type.regular_plugin import RegularPlugin
 
-SerializableType = TypeVar('SerializableType')
+SerializableType = TypeVar('SerializableType', bound=Serializable)
 
 
 class ServerInterface:
@@ -53,7 +54,7 @@ class ServerInterface:
 	def __init__(self, mcdr_server: 'MCDReforgedServer'):
 		self._mcdr_server = mcdr_server
 		if type(self) is ServerInterface:
-			# singleton, should only occurs during MCDReforgedServer construction
+			# singleton, should only occur during MCDReforgedServer construction
 			if ServerInterface.__global_instance is not None:
 				self._mcdr_server.logger.warning('Double assigning the singleton instance in {}'.format(self.__class__.__name__), stack_info=True)
 			ServerInterface.__global_instance = self
@@ -112,7 +113,7 @@ class ServerInterface:
 
 		Using this method instead of :meth:`tr` allows you to display your texts in :ref:`user's preferred language <preference-language>` automatically
 
-		Of course you can construct :class:`~mcdreforged.translation.translation_text.RTextMCDRTranslation` yourself instead of using this method if you want
+		Of course, you can construct :class:`~mcdreforged.translation.translation_text.RTextMCDRTranslation` yourself instead of using this method if you want
 
 		:param translation_key: The key of the translation
 		:param args: The args to be formatted
@@ -301,7 +302,11 @@ class ServerInterface:
 
 	def execute(self, text: str, *, encoding: Optional[str] = None) -> None:
 		"""
-		Execute a command by sending the command content to server's standard input stream
+		Execute a server command by sending the command content to server's standard input stream
+
+		.. seealso::
+
+			:meth:`execute_command` if you want to execute command in MCDR's command system
 
 		:param text: The content of the command you want to send
 		:param encoding: The encoding method for the text.
@@ -462,7 +467,7 @@ class ServerInterface:
 
 	def get_all_metadata(self) -> Dict[str, Metadata]:
 		"""
-		Return a dict containing metadatas of all loaded plugin with (plugin_id, metadata) as key-value pair
+		Return a dict containing metadata of all loaded plugin with (plugin_id, metadata) as key-value pair
 		"""
 		result = {}
 		for plugin in self._mcdr_server.plugin_manager.get_all_plugins():
@@ -475,44 +480,36 @@ class ServerInterface:
 
 	# Notes: All plugin manipulation will trigger a dependency check, which might cause unwanted plugin operations
 
-	def __check_if_success(self, operation_result: SingleOperationResult, check_loaded: bool) -> bool:
-		"""
-		Check if there's any plugin inside the given operation result (load result / reload result etc.)
-		Then check if the plugin passed the dependency check if param check_loaded is True
-		"""
-		success = operation_result.has_success()
-		if success and check_loaded:
-			plugin = operation_result.success_list[0]
-			success = plugin in self._mcdr_server.plugin_manager.last_operation_result.dependency_check_result.success_list
-		return success
-
-	def __not_loaded_regular_plugin_manipulate(self, plugin_file_path: str, handler: Callable[['PluginManager'], Callable[[str], Any]]) -> bool:
+	def __not_loaded_regular_plugin_manipulate(self, plugin_file_path: str, handler: Callable[['PluginManager'], Callable[[str], Future[PluginOperationResult]]]) -> bool:
 		"""
 		Manipulate a not loaded regular plugin from a given file path
 		:param plugin_file_path: The path to the not loaded new plugin
 		:param handler: What you want to do with Plugin Manager to the given file path
 		:return: If success
 		"""
-		self._mcdr_server.plugin_manager.last_operation_result.clear()
-		handler(self._mcdr_server.plugin_manager)(plugin_file_path)
-		return self.__check_if_success(self._mcdr_server.plugin_manager.last_operation_result.load_result, check_loaded=True)  # the operations is always loading a plugin
+		future: Future[PluginOperationResult] = handler(self._mcdr_server.plugin_manager)(plugin_file_path)
+		if future.is_finished():
+			return future.get().get_if_success(PluginResultType.LOAD)  # the operations are always loading a plugin
+		else:
+			return False  # TODO handle unknown result caused by chained sync plugin operation
 
-	def __existed_regular_plugin_manipulate(self, plugin_id: str, handler: Callable[['PluginManager'], Callable[['RegularPlugin'], Any]], result_getter: Callable[[PluginOperationResult], SingleOperationResult], check_loaded: bool) -> bool or None:
+	def __existed_regular_plugin_manipulate(self, plugin_id: str, handler: Callable[['PluginManager'], Callable[['RegularPlugin'], Any]],result_type: PluginResultType) -> Optional[bool]:
 		"""
 		Manipulate a loaded regular plugin from a given plugin id
 		:param plugin_id: The plugin id of the plugin you want to manipulate
 		:param handler: What callable you want to use with Plugin Manager to the plugin id,
 		the returned callable accepts the plugin instance
-		:param result_getter: How to get the single operation result from the plugin operation result.
+		:param result_type: The type of the result. It's used to determine how to get the single operation result from the plugin operation result.
 		It's used to determine if the operation succeeded
 		:return: If success, None if plugin not found
 		"""
 		plugin = self._mcdr_server.plugin_manager.get_regular_plugin_from_id(plugin_id)
 		if plugin is not None:
-			self._mcdr_server.plugin_manager.last_operation_result.clear()
-			handler(self._mcdr_server.plugin_manager)(plugin)
-			opt_result = result_getter(self._mcdr_server.plugin_manager.last_operation_result)
-			return self.__check_if_success(opt_result, check_loaded)
+			future = handler(self._mcdr_server.plugin_manager)(plugin)
+			if future.is_finished():
+				return future.get().get_if_success(result_type)
+			else:
+				return None
 		return None
 
 	def load_plugin(self, plugin_file_path: str) -> bool:
@@ -540,7 +537,7 @@ class ServerInterface:
 		:param plugin_id: The id of the plugin to reload. Example: ``"my_plugin"``
 		:return: A bool indicating if the plugin gets reloaded successfully, or None if plugin not found
 		"""
-		return self.__existed_regular_plugin_manipulate(plugin_id, lambda mgr: mgr.reload_plugin, lambda lor: lor.reload_result, check_loaded=True)
+		return self.__existed_regular_plugin_manipulate(plugin_id, lambda mgr: mgr.reload_plugin, PluginResultType.RELOAD)
 
 	def unload_plugin(self, plugin_id: str) -> Optional[bool]:
 		"""
@@ -549,7 +546,7 @@ class ServerInterface:
 		:param plugin_id: The id of the plugin to unload. Example: ``"my_plugin"``
 		:return: A bool indicating if the plugin gets unloaded successfully, or None if plugin not found
 		"""
-		return self.__existed_regular_plugin_manipulate(plugin_id, lambda mgr: mgr.unload_plugin, lambda lor: lor.unload_result, check_loaded=False)
+		return self.__existed_regular_plugin_manipulate(plugin_id, lambda mgr: mgr.unload_plugin, PluginResultType.UNLOAD)
 
 	def disable_plugin(self, plugin_id: str) -> Optional[bool]:
 		"""
@@ -558,7 +555,7 @@ class ServerInterface:
 		:param plugin_id: The id of the plugin to disable. Example: ``"my_plugin"``
 		:return: A bool indicating if the plugin gets disabled successfully, or None if plugin not found
 		"""
-		return self.__existed_regular_plugin_manipulate(plugin_id, lambda mgr: mgr.disable_plugin, lambda lor: lor.unload_result, check_loaded=False)
+		return self.__existed_regular_plugin_manipulate(plugin_id, lambda mgr: mgr.disable_plugin, PluginResultType.UNLOAD)
 
 	def refresh_all_plugins(self) -> None:
 		"""
@@ -600,13 +597,67 @@ class ServerInterface:
 		:param on_executor_thread: By default the event will be dispatched in a new task in task executor thread
 			If it's set to False. The event will be dispatched immediately
 		"""
-		misc_util.check_type(event, PluginEvent)
+		class_util.check_type(event, PluginEvent)
 		if MCDRPluginEvents.contains_id(event.id):
 			raise ValueError('Cannot dispatch event with already exists event id {}'.format(event.id))
 		self._mcdr_server.plugin_manager.dispatch_event(event, args, on_executor_thread=on_executor_thread)
 
 	# ------------------------
-	#		Permission
+	#      Configuration
+	# ------------------------
+
+	def get_mcdr_language(self) -> str:
+		"""
+		Return the current language MCDR is using
+		"""
+		return self._mcdr_server.translation_manager.language
+
+	def get_mcdr_config(self) -> dict:
+		"""
+		Return the current config of MCDR as a dict
+		"""
+		return self._mcdr_server.config.to_dict()
+
+	def modify_mcdr_config(self, changes: Dict[Union[Tuple[str], str], Any]):
+		"""
+		Modify the configuration of MCDR
+
+		The modification will be written to the disk and take effect immediately
+
+		Currently, MCDR will not validate the type of the value
+
+		Example usages::
+
+			server.modify_mcdr_config({'encoding': 'utf8'})
+			server.modify_mcdr_config({'rcon.address': '127.0.0.1', 'rcon.port': 23000})
+			server.modify_mcdr_config({('debug', 'command'): True})
+
+		:param changes:
+
+			A dict storing the changes to the config. For the entries of the dict:
+			The key can be a tuple storing the path to the config value, or a str that concat the path with ``"."``;
+			The value is the config value to be set
+
+		.. versionadded:: v2.7.0
+		"""
+		self._mcdr_server.config.set_values(changes)
+		self._mcdr_server.config.save()
+		self._mcdr_server.on_config_changed(log=False)
+
+	def reload_config_file(self, *, log: bool = False):
+		"""
+		Reload the configuration of MCDR from config file
+
+		It has the same effect as command ``!!MCDR reload config``
+
+		:keyword log: If the config changing messages should be logged
+
+		.. versionadded:: v2.7.0
+		"""
+		self._mcdr_server.load_config(log=log)
+
+	# ------------------------
+	#       Permission
 	# ------------------------
 
 	def get_permission_level(self, obj: Union[str, Info, CommandSource]) -> int:
@@ -645,8 +696,18 @@ class ServerInterface:
 			raise TypeError('Parameter level needs to be a permission related value')
 		self._mcdr_server.permission_manager.set_permission_level(player, level)
 
+	def reload_permission_file(self):
+		"""
+		Reload the permission of MCDR from permission file
+
+		It has the same effect as command ``!!MCDR reload permission``
+
+		.. versionadded:: v2.7.0
+		"""
+		self._mcdr_server.permission_manager.load_permission_file()
+
 	# ------------------------
-	#		 Command
+	#         Command
 	# ------------------------
 
 	def get_plugin_command_source(self) -> PluginCommandSource:
@@ -659,7 +720,11 @@ class ServerInterface:
 
 	def execute_command(self, command: str, source: CommandSource = None) -> None:
 		"""
-		Execute a single command
+		Execute a single command in MCDR's command system
+
+		.. seealso::
+
+			:meth:`execute` if you want to send some text to server's standard input stream
 
 		:param command: The command you want to execute
 		:param source: The command source that is used to execute the command. If it's not specified MCDR
@@ -667,8 +732,8 @@ class ServerInterface:
 		"""
 		if source is None:
 			source = self.get_plugin_command_source()
-		misc_util.check_type(command, str)
-		misc_util.check_type(source, CommandSource)
+		class_util.check_type(command, str)
+		class_util.check_type(source, CommandSource)
 		self._mcdr_server.command_manager.execute_command(command, source)
 
 	# ------------------------
@@ -711,18 +776,6 @@ class ServerInterface:
 		"""
 		return self._mcdr_server.rcon_manager.send_command(command)
 
-	def get_mcdr_language(self) -> str:
-		"""
-		Return the current language MCDR is using
-		"""
-		return self._mcdr_server.translation_manager.language
-
-	def get_mcdr_config(self) -> dict:
-		"""
-		Return the current config of MCDR as a dict
-		"""
-		return self._mcdr_server.config.to_dict()
-
 	def schedule_task(self, callable_: Callable[[], Any], *, block: bool = False, timeout: Optional[float] = None) -> None:
 		"""
 		Schedule a callback task to be run in task executor thread
@@ -922,10 +975,8 @@ class PluginServerInterface(ServerInterface):
 			if echo_in_console and not (source_to_reply is not None and source_to_reply.is_console):
 				self.logger.info(msg)
 
-		if target_class is not None:
-			target_class: Type[Serializable]
-			if default_config is None:
-				default_config = target_class.get_default().serialize()
+		if target_class is not None and default_config is None:
+			default_config = target_class.get_default().serialize()
 		config_file_path = os.path.join(self.get_data_folder(), file_name) if in_data_folder else file_name
 		needs_save = False
 		try:
@@ -943,9 +994,7 @@ class PluginServerInterface(ServerInterface):
 			if default_config is not None:
 				# constructing the result config based on the given default config
 				for key, value in default_config.items():
-					if key in read_data:
-						result_config[key] = read_data[key]
-					else:
+					if key not in read_data:
 						result_config[key] = value
 						log(self._mcdr_server.tr('server_interface.load_config_simple.key_missed', key, value))
 						needs_save = True
@@ -990,6 +1039,6 @@ class PluginServerInterface(ServerInterface):
 		target_folder = os.path.dirname(config_file_path)
 		if len(target_folder) > 0 and not os.path.isdir(target_folder):
 			os.makedirs(target_folder)
-		with open(config_file_path, 'w', encoding=encoding) as file:
+		with file_util.safe_write(config_file_path, encoding=encoding) as file:
 			# config file should be nicely readable, so here come the indent and non-ascii chars
 			json.dump(data, file, indent=4, ensure_ascii=False)
