@@ -19,3 +19,12 @@ class IllegalStateError(RuntimeError):
 
 class ServerStopped(RuntimeError):
 	pass
+
+
+class IllegalPluginStructure(Exception):
+	pass
+
+
+# When the metadata of a plugin is invalid
+class BrokenMetadata(Exception):
+	pass
