@@ -3,7 +3,7 @@ Misc tool collection
 """
 import inspect
 import threading
-from typing import List, Callable, Tuple, TypeVar, Any, Type, Optional, Union, Iterable
+from typing import List, Callable, Tuple, TypeVar, Any, Optional, Iterable
 
 
 def start_thread(func: Callable, args: Tuple, name: Optional[str] = None):
@@ -19,42 +19,28 @@ def unique_list(lst: Iterable[T]) -> List[T]:
 	return list(dict.fromkeys(lst).keys())
 
 
+def deep_copy_dict(source: dict) -> dict:
+	ret = {}
+	for key, value in source.items():
+		if isinstance(value, dict):
+			value = deep_copy_dict(value)
+		ret[key] = value
+	return ret
+
+
 def print_text_to_console(logger, text: Any):
 	from mcdreforged.minecraft.rtext.text import RTextBase
 	for line in RTextBase.from_any(text).to_colored_text().splitlines():
 		logger.info(line)
 
 
-def check_type(value: Any, types: Union[Type, Iterable[Type]], error_message: str = None):
-	if not isinstance(types, Iterable):
-		types = [types]
-	if not any(map(lambda t: isinstance(value, t), types)):
-		if error_message is None:
-			error_message = 'Except type {} but found type {}'.format(types, type(value))
-		raise TypeError(error_message)
-
-
 def copy_signature(target: Callable, origin: Callable) -> Callable:
 	"""
 	Copy the function signature of origin into target
 	"""
+	assert callable(target) and callable(origin)
+
 	# https://stackoverflow.com/questions/39926567/python-create-decorator-preserving-function-arguments
 	target.__signature__ = inspect.signature(origin)
 	return target
 
-
-class WaitableCallable:
-	def __init__(self, func: Callable):
-		self.__func = func
-		self.__event = threading.Event()
-
-	def __call__(self, *args, **kwargs):
-		rv = self.__func(*args, **kwargs)
-		self.__event.set()
-		return rv
-
-	def wait(self, timeout: Optional[float] = None) -> bool:
-		"""
-		Return if the event has been set
-		"""
-		return self.__event.wait(timeout=timeout)
