@@ -10,12 +10,12 @@ __all__ = [
 	'get_data', 'get_yaml'
 ]
 
-from ruamel import yaml
+from ruamel.yaml import YAML
 from ruamel.yaml.comments import CommentedMap
 
-from mcdreforged.constant import PACKAGE_NAME
+from mcdreforged.constants import core_constant
 
-ROOT_PACKAGE = PACKAGE_NAME
+ROOT_PACKAGE = core_constant.PACKAGE_NAME
 
 
 def __get_path(path: str) -> str:
@@ -32,7 +32,8 @@ def get_yaml(path: str) -> CommentedMap:
 	bytes_data = get_data(path)
 	# Replace CRLF or yaml loader will load extra lines
 	string_data = bytes_data.decode('utf8').replace('\r\n', '\n')
-	return yaml.round_trip_load(string_data)
+	ret = YAML().load(string_data)
+	return ret
 
 
 if __name__ == '__main__':
